package com.jjkj.hikvison;

import cn.hutool.core.date.DateUtil;
import com.sun.jna.NativeLong;
import com.sun.jna.ptr.IntByReference;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.File;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

public class HikvisionUtil {
    static HCNetSDK hCNetSDK = HCNetSDK.INSTANCE;             //sdk对象实例
    static HCNetSDK.NET_DVR_DEVICEINFO_V30 m_strDeviceInfo;   //设备信息
    static HCNetSDK.NET_DVR_JPEGPARA picInfo;                 //图片信息
    static HCNetSDK.NET_DVR_CLIENTINFO playInfo;              //预览信息
    static NativeLong lUserID;                                //用户句柄
    static NativeLong lPreviewHandle;                         //预览句柄
    public static Map lPreviewHandleMap = new HashMap(); //定义map
    private static final Logger logger = LoggerFactory.getLogger(HikvisionUtil.class);
    private static SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

    /**
     * 函数功能说明：sdk初始化    DvrUtil.RECORD_SAVE_PATH
     * 创建者名字：周叶平
     * 创建日期：  2015-1-20 下午4:08:58
     *
     * @return boolean
     * @throws
     * @参数： @return
     */

    public static boolean init() {
        boolean res = hCNetSDK.NET_DVR_Init();

        boolean res1 = hCNetSDK.NET_DVR_SetConnectTime(2000, 1);
        boolean res2 = hCNetSDK.NET_DVR_SetReconnect(10000, true);
        if (res1 && res2) {
            //    System.out.println("连接成功！");
        }
        hCNetSDK.NET_DVR_SetLogToFile(true, "D:\\log", true);
        return res;
    }

    public static NativeLong login(String idaddress, short iport, String username, String password) {
        if (init()) {
            m_strDeviceInfo = new HCNetSDK.NET_DVR_DEVICEINFO_V30();
            lUserID = hCNetSDK.NET_DVR_Login_V30(idaddress, iport, username, password, m_strDeviceInfo);
            logger.info("hksdk(视频)-登录海康录像机信息,状态值:" + hCNetSDK.NET_DVR_GetLastError());
            int errorCode = hCNetSDK.NET_DVR_GetLastError();
            if (errorCode == 52) { // 录像机 登录超过最大限值
                hCNetSDK.NET_DVR_Logout(lUserID);//退出录像机
                hCNetSDK.NET_DVR_Init();
                logger.info("hksdk(视频)-达到最大数退出后状态值:" + hCNetSDK.NET_DVR_GetLastError());
                lUserID = hCNetSDK.NET_DVR_Login_V30(idaddress, iport, username, password, m_strDeviceInfo);
            }
        }
        return lUserID;
    }

    /**
     * 函数功能说明： dvr登出
     * 创建者名字：周叶平
     * 创建日期：  2015-1-20 下午5:30:13
     *
     * @return boolean
     * @throws
     * @参数： @param id
     * @参数： @return
     */
    public static boolean logout(NativeLong id) {
        boolean s = false;
        if (new NativeLong(0).intValue() == id.intValue()) {
            s = hCNetSDK.NET_DVR_Logout_V30(id);
            if (s) {
                hCNetSDK.NET_DVR_Cleanup();   //释放sdk
            }
        }
        return s;
    }

    /**
     * 函数功能说明： 抓图
     * 创建者名字：ngw
     * 创建日期：  2015-1-20 下午4:34:36
     *
     * @return string     文件绝对路径
     * @throws
     * @参数： @param idaddress  ip地址
     * @参数： @param iport      端口号
     * @参数： @param username   用户名
     * @参数： @param password   密码
     * @参数： @return
     */
    public static String catchHikvisionPicture(String fileName, String idaddress, short iport, String username, String password, int chanNum, String imagePath) throws Exception {
        NativeLong lUserID11; //增加登录情况赋值
        HCNetSDK.NET_DVR_JPEGPARA picInfo11;                 //图片信息
        String res = null;
        if (imagePath.indexOf(":") < 0) {
            imagePath += ":";
        }
        imagePath = imagePath + "/" + DateUtil.format(new Date(), "yyyyMMdd");
        File savepath = new File(imagePath);
        if (!savepath.exists()) {
            savepath.mkdirs();
        } else {
            if (!savepath.isDirectory()) {
                savepath.delete();
                savepath.mkdirs();
            }
        }
        lUserID11 = login(idaddress, iport, username, password);
        picInfo11 = new HCNetSDK.NET_DVR_JPEGPARA();
        picInfo11.wPicQuality = 0;   //图片质量
        picInfo11.wPicSize = 0xff;   //图片大小
        String path = imagePath + "/" + fileName + ".jpeg";
        boolean s = hCNetSDK.NET_DVR_CaptureJPEGPicture(lUserID11, new NativeLong(chanNum), picInfo11, path);
        if (s) {
            res = "success@" + path;
            logger.info(path + "录像机抓取照片成功");

        } else {

            Integer code = hCNetSDK.NET_DVR_GetLastError();
            String message = NvrErrorMsg(code);
            message = "海康录像机抓拍报错信息&状态码:" + code + " &:" + message;
            logger.info("海康录像机抓拍报错信息&状态码:" + code + " &:" + message);
            res = message;
        }
//        释放资源后退出登录
        logger.info("登录状态==" + DateUtil.format(new Date(), "yyyy-MM-dd HH:mm:ss") + "====" + lUserID11);
        hCNetSDK.NET_DVR_Cleanup();
        hCNetSDK.NET_DVR_Logout(lUserID11);
        logger.info(DateUtil.format(new Date(), "yyyy-MM-dd HH:mm:ss") + "====" + "---注销代码走完---");
        logout(lUserID11);

        return res;

    }

    /**
     * 函数功能说明： 按时间下载视频
     * 创建者名字： 南若飞
     *
     *
     * @param ip
     * @param
     * @param
     * @param port
     * @param userName
     * @param channel
     * @param passowrd
     * @param startTime
     * @param endTime
     * @return
     */
    public static boolean downloadVideo(String ip,  String savePath, String fileName,short port, String userName, int channel, String passowrd, Date startTime, Date endTime) {
        NativeLong lUserID11 = null;
        File savepath = new File(savePath);
        if (!savepath.exists()) {
            savepath.mkdirs();
        }
        hCNetSDK.NET_DVR_Cleanup();   //释放sdk
        boolean initFlag = hCNetSDK.NET_DVR_Init();
        if (!initFlag) { //返回值为布尔值 fasle初始化失败
            logger.error("hksdk(视频)-海康sdk初始化失败!");
            return false;
        }
        HCNetSDK.NET_DVR_DEVICEINFO_V30 deviceInfo = new HCNetSDK.NET_DVR_DEVICEINFO_V30();
        lUserID11 = hCNetSDK.NET_DVR_Login_V30(ip, port, userName, passowrd, deviceInfo);
        logger.info("hksdk(视频)-登录海康录像机信息,状态值:" + hCNetSDK.NET_DVR_GetLastError());
        int errorCode = hCNetSDK.NET_DVR_GetLastError();
        if (errorCode == 52) { // 录像机 登录超过最大限值
            hCNetSDK.NET_DVR_Logout(lUserID11);//退出录像机
            hCNetSDK.NET_DVR_Init();
            logger.info("hksdk(视频)-达到最大数退出后状态值:" + hCNetSDK.NET_DVR_GetLastError());
            lUserID11 = hCNetSDK.NET_DVR_Login_V30(ip, port, userName, passowrd, deviceInfo);
        }
        long lUserId = lUserID11.longValue();
        if (lUserId == -1) {
            logger.warn("hksdk(视频)-海康sdk登录失败!");
            return false;
        }
        NativeLong loadHandle = new NativeLong(-1);
        if (loadHandle.intValue() == -1) {
            /*** 设置保存路径 ***/
            String sFileName = savepath + "/" + fileName; // 下载路径+文件名
            loadHandle = hCNetSDK.NET_DVR_GetFileByTime(lUserID11, new NativeLong(channel), getHkTime(startTime), getHkTime(endTime), sFileName);
            logger.info("hksdk(视频)-获取播放句柄信息,状态值:" + hCNetSDK.NET_DVR_GetLastError());
            if (loadHandle.intValue() >= 0) {
                boolean downloadFlag = hCNetSDK.NET_DVR_PlayBackControl(loadHandle, hCNetSDK.NET_DVR_PLAYSTART, 0, null);
                int tmp = -1;
                IntByReference pos = new IntByReference();
                while (true) {
                    boolean backFlag = hCNetSDK.NET_DVR_PlayBackControl(loadHandle, hCNetSDK.NET_DVR_PLAYGETPOS, 0, pos);
                    if (!backFlag) {//防止单个线程死循环
                        return downloadFlag;
                    }
                    int produce = pos.getValue();
                    if ((produce % 10) == 0 && tmp != produce) {//输出进度
                        tmp = produce;
                        logger.info("hksdk(视频)-视频下载进度:" + "==" + produce + "%");
                    }
                    if (produce == 100) {//下载成功
                        hCNetSDK.NET_DVR_StopGetFile(loadHandle);
                        loadHandle.setValue(-1);
                        hCNetSDK.NET_DVR_Logout(lUserID11);//退出录像机
                        logger.info("hksdk(视频)-退出状态" + hCNetSDK.NET_DVR_GetLastError());
                        //hcNetSDK.NET_DVR_Cleanup();
                        return true;
                    }
                    if (produce > 100) {//下载失败
                        hCNetSDK.NET_DVR_StopGetFile(loadHandle);
                        loadHandle.setValue(-1);
                        logger.warn("hksdk(视频)-海康sdk由于网络原因或DVR忙,下载异常终止!错误原因:" + hCNetSDK.NET_DVR_GetLastError());
                        hCNetSDK.NET_DVR_Logout(lUserID11);//退出录像机
                        logger.info("hksdk(视频)-退出状态" + hCNetSDK.NET_DVR_GetLastError());
                        return false;
                    }
                }
            } else {
                logger.info("hksdk(视频)-下载失败" + hCNetSDK.NET_DVR_GetLastError());
                hCNetSDK.NET_DVR_Logout(lUserID11);//退出录像机
                return false;
            }
        }
        return false;
    }
    /**
     * 函数功能说明： 获取海康录像机格式的时间
     * 创建者名字： 南若飞
     *
     *
     * @param time
     * @return
     */
    public static HCNetSDK.NET_DVR_TIME getHkTime(Date time) {
        HCNetSDK.NET_DVR_TIME structTime = new HCNetSDK.NET_DVR_TIME();
        String str = new SimpleDateFormat("yyyy-MM-dd-HH-mm-ss").format(time);
        String[] times = str.split("-");
        structTime.dwYear = Integer.parseInt(times[0]);
        structTime.dwMonth = Integer.parseInt(times[1]);
        structTime.dwDay = Integer.parseInt(times[2]);
        structTime.dwHour = Integer.parseInt(times[3]);
        structTime.dwMinute = Integer.parseInt(times[4]);
        structTime.dwSecond = Integer.parseInt(times[5]);
        return structTime;
    }

    /**
     * 获取错误原因
     *
     * @param code 错误码
     */
    public static String NvrErrorMsg(Integer code) {
        switch (code) {
            case 1:
                return "用户名密码错误。注册时输入的用户名或者密码错误";
            case 2:
                return "权限不足。一般和通道相关";
            case 3:
                return "SDK未初始化。必须先调用NET_DVR_Init，该接口是调用其他SDK函数的前提且一个程序进程只需要调用一次。";
            case 5:
                return "连接到设备的用户个数超过最大";
            case 6:
                return "版本不匹配。SDK和设备的版本不匹配。";
            case 4:
                return "通道号错误。设备通道分模拟通道和数字通道（IP通道）";
            case 7:
                return "连接设备失败。设备不在线或网络原因引起的连接超时等";
            case 8:
                return "向设备发送失败";
            case 9:
                return "从设备接收数据失败";
            case 10:
                return "从设备接收数据超时";
            case 11:
                return "传送的数据有误。发送给设备或者从设备接收到的数据错误。";
            case 14:
                return "设备命令执行超时";
            case 17:
                return "参数错误。SDK接口中给入的输入或输出参数为空";
            case 18:
                return "设备通道处于错误状态";
            case 19:
                return "设备无硬盘。当设备无硬盘时，对设备的录像文件、硬盘配置等操作失败";
            case 20:
                return "硬盘号错误。当对设备进行硬盘管理操作时，指定的硬盘号不存在时返回该错误。";
            case 21:
                return "设备硬盘满";
            case 22:
                return "设备硬盘出错";
            case 23:
                return "设备不支持";
            case 29:
                return "设备操作失败";
            case 34:
                return "创建文件出错。本地录像、保存图片、获取配置文件和远程下载录像时创建文件失败。";
            case 41:
                return "SDK资源分配错误";
            case 47:
                return "用户不存在。注册的用户ID已注销或不可用";
            case 52:
                return "登录设备的用户数达到最大。";
            case 61:
                return "图像素位数不符，限24色。";
            case 62:
                return "图片高*宽超限，限128*256";
            case 63:
                return "图片大小超限，限100K。";
            case 72:
                return "绑定套接字失败";
            case 73:
                return "socket连接中断，此错误通常是由于连接中断或目的地不可达";
            case 91:
                return "通道不支持该操作";
            case 174:
                return "IP冲突。";
        }
        return null;
    }

    public static void main(String[] args) throws Exception {
//        String path = catchHikvisionPicture("test", "192.168.0.98", (short) 8000, "admin",
//                "jjkj2012", 34, "d:/jczp");
//        System.out.println(path);
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
//        boolean flag = DvrUtil.getInstance().downloadVideo("192.168.0.98", "E:/video/",
//                "12341.mp4", (short) 8000, "admin", 33, "jjkj2012",
//                sdf.parse("2024-02-19 14:20:00"), sdf.parse("2024-02-19 14:22:00"));
//        System.out.println(flag);
        downloadVideo("192.168.0.98","d:/jczp","my.mp4",(short)8000,"admin",34,
                "jjkj2012",  sdf.parse("2024-05-11 08:55:39"), sdf.parse("2024-05-11 08:55:59"));
//

    }
}
