package com.jjkj.util;

import java.sql.Timestamp;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.Locale;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class DateUtil {
    public static final String FULL = "yyyy-MM-dd HH:mm:ss";
    public static final String FULL_SPRIT = "yyyy/MM/dd HH:mm:ss";
    public static final String FULL_ZH = "yyyy年MM月dd日 HH时mm分ss秒";

    public static final String FULL_ZH_YEAR = "yyyy年MM月dd日 HH:mm:ss";
    public static final String FULL_ZH_NOSECOND = "yyyy年MM月dd日 HH:mm";

    public static final String YEAR = "yyyy-MM-dd";
    public static final String YEAR_SPRIT = "yyyy/MM/dd";
    public static final String YEAR_ZH = "yyyy年MM月dd日";
    public static final String YEAR_MONTH = "yyyy-MM";

    /**
     * 日期转字符串
     *
     * @param date       日期对象
     * @param formartStr 格式化字符串
     * @return 根据格式化字符串转换后的日期字符串
     */
    public static String dateToString(Date date, String formartStr) {
        String strDate = null;
        DateUtil ub = new DateUtil();

        if ((formartStr != null) && (!"".equals(formartStr))) {
            SimpleDateFormat sdf = new SimpleDateFormat(formartStr);
            strDate = sdf.format(date);
        }
        return strDate;
    }

    /**
     * 字符串转日期
     *
     * @param strDate    日期字符串
     * @param formartStr 格式化字符串
     * @return 根据格式化字符串转换后的日期对象
     */
    public static Date stringToDate(String strDate, String formartStr) {
        Date date = null;
        if ((formartStr != null) && (!"".equals(formartStr))) {
            SimpleDateFormat sdf = new SimpleDateFormat(formartStr);
            try {
                if (strDate != null && !"".equals(strDate)) {
                    date = sdf.parse(strDate);
                }
            } catch (ParseException e) {
                date = null;
                e.printStackTrace();
            }
        }
        return date;
    }

    /**
     * 获得当前时间
     *
     * @param formartStr 格式化字符串
     * @return 根据格式化字符串转换后的当前时间字符串
     */
    public static String nowTime(String formartStr) {
        String strDate = null;
        DateUtil ub = new DateUtil();
        if ((formartStr != null) && (!"".equals(formartStr))) {
            SimpleDateFormat sdf = new SimpleDateFormat(formartStr);
            strDate = sdf.format(new Date());
        }
        return strDate;
    }

    /**
     * 检查日期是否在今天
     *
     * @param date 要检查的日期对象
     * @return 如果在今天返回ture，不在今天返回false
     */
    public static boolean checkDateInToday(Date date) {
        if (date == null) {
            return true;
        }
        boolean flag = false;
        Date now = new Date();

        String nowStr = dateToString(now, "yyyy-MM-dd");
        String dateStr = dateToString(date, "yyyy-MM-dd");

        if (!nowStr.equals(dateStr)) {
            flag = true;
        }

        return flag;
    }

    /**
     * 函数功能说明： 获得当前月第一天 创建者名字：周叶平 创建日期： 2014-7-31 下午3:59:43
     *
     * @return String
     * @throws
     * @参数： @return
     */
    public static String getFirstDayOfMonth() {
        Calendar c = Calendar.getInstance();
        c.set(Calendar.DAY_OF_MONTH, 1);// 设置为1号,当前日期既为本月第一天
        return dateToString(c.getTime(), YEAR);

    }

    /**
     * 函数功能说明：获得当月最后一天 创建者名字：周叶平 创建日期： 2014-7-31 下午3:40:51
     *
     * @return String
     * @throws
     * @参数： @return
     */
    public static String getLastDayOfMonth() {
        Calendar ca = Calendar.getInstance();
        ca.set(Calendar.DAY_OF_MONTH, ca
                .getActualMaximum(Calendar.DAY_OF_MONTH));
        return dateToString(ca.getTime(), YEAR);
    }

    /**
     * 函数功能说明：获得当前日期的前一个月 创建者名字：周叶平 创建日期： 2014-11-13 上午9:12:51
     *
     * @return String yyyy-mm
     * @throws
     * @参数： @return
     */
    public static String getBeforeMonth() {
        Calendar ca = Calendar.getInstance();
        ca.set(Calendar.MONTH, ca.get(Calendar.MONTH) - 1);
        return dateToString(ca.getTime(), YEAR_MONTH);
    }

    /**
     * 根据day值获取day天前的日期值
     *
     * @return
     */
    public static String getBeforeHarfMonth(Integer day) {
        Date date = new Date();
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        System.out.println("当前时间为：" + sdf.format(date));

        Calendar cal = Calendar.getInstance();
        //获取前面的时间用-负号
        cal.setTime(date);
        cal.add(Calendar.DAY_OF_MONTH, day);
        System.out.println(day+"天前的时间为：" + sdf.format(cal.getTime()));
        return sdf.format(cal.getTime());
    }

    /**
     * 函数功能说明：获得当月后推一年的第一个月 创建者名字：周叶平 创建日期： 2014-11-18 上午10:52:20
     *
     * @return String
     * @throws
     * @参数： @return
     */
    public static String getBeforeMonthOfYear() {
        Calendar ca = Calendar.getInstance();
        ca.set(Calendar.MONTH, ca.get(Calendar.MONTH) - 11);
        return dateToString(ca.getTime(), YEAR_MONTH);
    }

    public static boolean isDate(String str2) {
        boolean flag = false;
        String pat = "\\d{4}-\\d{2}-\\d{2}"; // 指定好正则表达式
        Pattern p = Pattern.compile(pat); // 实例化Pattern类
        Matcher m = p.matcher(str2); // 实例化Matcher类
        if (m.matches()) { // 进行验证的匹配，使用正则
            flag = true;
        } else {
            flag = false;

        }
        return flag;

    }

    /**
     * 比较两个时间差
     **/
    public static long getBetweenHours(String beginDate, String endDate) {
        SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        long hours = 0l;
        Date beginDate11 = null, endDate11 = null;
        try {

            if (!"".equals(beginDate) && beginDate != null) {

                beginDate11 = df.parse(beginDate);
            }
            if (!"".equals(endDate) && endDate != null) {
                endDate11 = df.parse(endDate);
            }
            long chazhi = endDate11.getTime() - beginDate11.getTime();

            long day = chazhi / (24 * 60 * 60 * 1000);
            hours = (chazhi / (60 * 60 * 1000) - day * 24);
            // hours=chazhi%(24*3600)/3600;
        } catch (Exception ce) {
            ce.printStackTrace();
        }
        return hours;
    }

    /**
     * 获取两个时间差 天数
     */
    public static int differentDays(Date date1, Date date2) {
        if (date1.after(date2)) {
            int c = (int) ((date1.getTime() - date2.getTime()) / (24 * 60 * 60 * 1000));
            return c;
        } else if (date2.after(date1)) {
            int c = (int) ((date2.getTime() - date1.getTime()) / (24 * 60 * 60 * 1000));
            return c;
        }
        return 0;
    }

    /**
     * 比较两个时间差是否超过半年
     * <p>
     * 赵驰
     **/
    public static long getBetweenmoth(String jcrq11, String jcrq) {
        /*
         * SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
         * long hours=0l; Date beginDate11=null,endDate11=null;
         */

        // SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        // Date jcrq111=null,jcrq22=null;

        long yxjcq = 01;
        try {
            // jcrq111=df.parse(jcrq1); jcrq22=df.parse(jcrq2);
            // String jcrq11=jcrq111.toString();
            // String jcrq=jcrq22.toString();
            if (!"".equals(jcrq11) && jcrq11 != null && !"".equals(jcrq)
                    && jcrq != null) {
                String beginStrnian = "", beginStryue = "", beginStrri = "", endStrnian = "", endStryue = "", endStrri = "";
                Integer begnian = 0, beyue = 0, beri = 0, endnian = 0, endyue = 0, endri = 0;
                if (jcrq11.length() >= 10) {
                    beginStrnian = jcrq11.substring(0, 4);
                    begnian = Integer.valueOf(beginStrnian);

                    beginStryue = jcrq11.substring(5, 7);
                    beyue = Integer.valueOf(beginStryue);

                    beginStrri = jcrq11.substring(8, 10);
                    beri = Integer.valueOf(beginStrri);
                }
                if (jcrq.length() >= 10) {
                    endStrnian = jcrq.substring(0, 4);
                    endnian = Integer.valueOf(endStrnian);

                    endStryue = jcrq.substring(5, 7);
                    endyue = Integer.valueOf(endStryue);

                    endStrri = jcrq.substring(8, 10);
                    endri = Integer.valueOf(endStrri);

                }
                if (beginStrnian.equals(endStrnian)) {
                    if (endri >= beri) {
                        yxjcq = endyue - beyue;
                    } else {
                        yxjcq = endyue - 1 - beyue;
                    }
                } else if (endnian > begnian) {
                    Integer nian = endnian - begnian;
                    yxjcq = (nian * 12 + endyue) - beyue;
                }

            }

            /*
             * if(!"".equals(beginDate) && beginDate!=null){
             *
             * beginDate11 = df.parse(beginDate);
             * System.out.println(beginDate11+"*****"); } if(!"".equals(endDate)
             * && endDate!=null){ endDate11 = df.parse(endDate);
             * System.out.println(endDate11+"*****"); }
             * System.out.println(beginDate11+"***"+endDate11); long
             * chazhi=endDate11.getTime()-beginDate11.getTime();
             *
             * long day=chazhi/(24*60*60*1000);
             * hours=(chazhi/(60*60*1000)-day*24);
             */
            // hours=chazhi%(24*3600)/3600;
        } catch (Exception ce) {
            ce.printStackTrace();
        }
        return yxjcq;
    }

    /**
     * 获取前后日期 i为正数 向后推迟i天，负数时向前提前i天
     *
     * @param i
     * @return
     */
    public static Date getdate(int i) {
        Date dat = null;
        Calendar cd = Calendar.getInstance();
        cd.add(Calendar.DATE, i);
        dat = cd.getTime();
        SimpleDateFormat dformat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        Timestamp date = Timestamp.valueOf(dformat.format(dat));
        return date;
    }

    /**
     * 获取当前时间
     *
     * @return
     */
    public static String getNowDate() {
        return new SimpleDateFormat(FULL).format(new Date());
    }

    /**
     * @throws ParseException
     */
    public static String getChinaDate(String s) throws ParseException {
        // String s = "Mon Feb 13 08:00:00 GMT+08:00 2012";
        // SimpleDateFormat sf = new
        // SimpleDateFormat("EEE MMM dd hh:mm:ss z yyyy", Locale.ENGLISH);
        s = "Sun Sep 02 2012 08:00:00 GMT+08:00";
        SimpleDateFormat sf = new SimpleDateFormat(
                "EEE MMM dd yyyy hh:mm:ss z", Locale.ENGLISH);
        Date date = sf.parse(s);
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy年MM月dd日");
        String result = sdf.format(date);
        return result;

    }

    public static String formattoString(Date date) throws Exception {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy年MM月dd日");
        return sdf.format(date);
    }

    /* 获取当前时间的前后N分钟 */
    public static Date getTimeBeforeN(int N) {
        Calendar c = Calendar.getInstance();
        c.add(Calendar.MINUTE, N);
        return c.getTime();
    }

    /* 判断当前时间是否在两个时间段之间，返回boolean */
    public static boolean judgeTime(String start, String end) {
        SimpleDateFormat sdf = new SimpleDateFormat("HH:mm");
        Date now = null;
        Date beginTime = null;
        Date endTime = null;
        if (start != null && !"".equals(start) && end != null
                && !"".equals(end)) {
            try {
                now = sdf.parse(sdf.format(new Date()));
                beginTime = sdf.parse(start);
                endTime = sdf.parse(end);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        boolean flag = belongCalendar(now, beginTime, endTime);
        return flag;
    }

    private static boolean belongCalendar(Date nowTime, Date beginTime,
                                          Date endTime) {
        Calendar date = Calendar.getInstance();
        date.setTime(nowTime);
        Calendar begin = Calendar.getInstance();
        begin.setTime(beginTime);
        Calendar end = Calendar.getInstance();
        end.setTime(endTime);
        if (date.after(begin) && date.before(end)) {
            return true;
        }
        return false;
    }

    public static String StringDateFormat(String time, String format) {
        SimpleDateFormat sf = new SimpleDateFormat(format);
        Date date = null;
        try {
            date = sf.parse(time);
        } catch (ParseException e) {
            e.printStackTrace();
        }
        return sf.format(date);
    }

    public static String getTimeFormat2TimeString(String timeFormat) throws Exception {
        return getmill2TimeString("" + getTimeFormat2mill(timeFormat));

    }

    /**
     * 将时间格式yyyy-MM-dd HH:mm:ss转成毫秒
     *
     * @param
     * @return
     * @throws ParseException
     * @throws Exception
     */
    public static Long getTimeFormat2mill(String timeStr) throws ParseException, Exception {
        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        Date date = simpleDateFormat.parse(timeStr);
        long ts = date.getTime();
        return ts;
    }

    /**
     * 将毫秒转成时间串yyyyMMddHHmmss
     *
     * @param
     * @return
     * @throws ParseException
     */
    public static String getmill2TimeString(String timeStr) throws ParseException {
        return getTimeFormat(timeStr).replaceAll("-", "").replaceAll(" ", "").replaceAll(":", "");
    }

    /**
     * 将毫秒字符串转成时间格式yyyy-MM-dd HH:mm:ss
     *
     * @param
     * @return
     */
    public static String getTimeFormat(String timeStr) {
        long time = Long.parseLong(timeStr);
        Calendar calendar = Calendar.getInstance();
        calendar.setTimeInMillis(time);
        int year = calendar.get(Calendar.YEAR);
        int month = calendar.get(Calendar.MONTH) + 1;
        String monthStr = addZero(month);
        int day = calendar.get(Calendar.DAY_OF_MONTH);
        String dayStr = addZero(day);
        int hour = calendar.get(Calendar.HOUR_OF_DAY);// 24小时制
        String hourStr = addZero(hour);
        int minute = calendar.get(Calendar.MINUTE);
        String minuteStr = addZero(minute);
        int second = calendar.get(Calendar.SECOND);
        String secondStr = addZero(second);
        return (year + "-" + monthStr + "-" + dayStr + " " + hourStr + ":" + minuteStr + ":" + secondStr);
    }

    private static String addZero(int param) {
        String paramStr = param < 10 ? "0" + param : "" + param;
        return paramStr;
    }

    public static void main(String[] args) {
        System.out.println(getdate(0));
        System.out.println(getdate(1));
    }
}