package com.jjkj.util;



import org.apache.log4j.Logger;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import java.io.*;
import java.math.BigDecimal;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.util.Base64;

/**
 * 文件处理工具类
 *
 * @author 南若飞
 */
public class FileUtils {
    private static Logger logger = Logger.getLogger(FileUtils.class);
    public static String FILENAME_PATTERN = "[a-zA-Z0-9_\\-\\|\\.\\u4e00-\\u9fa5]+";

    /**
     * 输出指定文件的byte数组
     *
     * @param filePath 文件路径
     * @param os       输出流
     * @return
     */
    public static void writeBytes(String filePath, OutputStream os) throws IOException {
        FileInputStream fis = null;
        try {
            File file = new File(filePath);
            if (!file.exists()) {
                throw new FileNotFoundException(filePath);
            }
            fis = new FileInputStream(file);
            byte[] b = new byte[1024];
            int length;
            while ((length = fis.read(b)) > 0) {
                os.write(b, 0, length);
            }
        } catch (IOException e) {
            throw e;
        } finally {
            os.close();
            fis.close();
        }
    }





    /**
     * 删除文件
     *
     * @param filePath 文件
     * @return
     */
    public static boolean deleteFile(String filePath) {
        boolean flag = false;
        File file = new File(filePath);
        // 路径为文件且不为空则进行删除
        if (file.isFile() && file.exists()) {
            flag = file.delete();
        }
        return flag;
    }

    /**
     * 文件名称验证
     *
     * @param filename 文件名称
     * @return true 正常 false 非法
     */
    public static boolean isValidFilename(String filename) {
        return filename.matches(FILENAME_PATTERN);
    }


    /**
     * 下载文件名重新编码
     *
     * @param request  请求对象
     * @param fileName 文件名
     * @return 编码后的文件名
     */
    public static String setFileDownloadHeader(HttpServletRequest request, String fileName) throws UnsupportedEncodingException {
        final String agent = request.getHeader("USER-AGENT");
        String filename = fileName;
        if (agent.contains("MSIE")) {
            // IE浏览器
            filename = URLEncoder.encode(filename, "utf-8");
            filename = filename.replace("+", " ");
        } else if (agent.contains("Firefox")) {
            // 火狐浏览器
            filename = new String(fileName.getBytes(), "ISO8859-1");
        } else if (agent.contains("Chrome")) {
            // google浏览器
            filename = URLEncoder.encode(filename, "utf-8");
        } else {
            // 其它浏览器
            filename = URLEncoder.encode(filename, "utf-8");
        }
        return filename;
    }

    /**
     * 下载文件名重新编码
     *
     * @param response     响应对象
     * @param realFileName 真实文件名
     */
    public static void setAttachmentResponseHeader(HttpServletResponse response, String realFileName) throws UnsupportedEncodingException {
        String percentEncodedFileName = percentEncode(realFileName);

        StringBuilder contentDispositionValue = new StringBuilder();
        contentDispositionValue.append("attachment; filename=")
                .append(percentEncodedFileName)
                .append(";")
                .append("filename*=")
                .append("utf-8''")
                .append(percentEncodedFileName);

        response.addHeader("Access-Control-Expose-Headers", "Content-Disposition,download-filename");
        response.setHeader("Content-disposition", contentDispositionValue.toString());
        response.setHeader("download-filename", percentEncodedFileName);
    }

    /**
     * 百分号编码工具方法
     *
     * @param s 需要百分号编码的字符串
     * @return 百分号编码后的字符串
     */
    public static String percentEncode(String s) throws UnsupportedEncodingException {
        String encode = URLEncoder.encode(s, StandardCharsets.UTF_8.toString());
        return encode.replaceAll("\\+", "%20");
    }

    /**
     * 获取图像后缀
     *
     * @param photoByte 图像数据
     * @return 后缀名
     */
    public static String getFileExtendName(byte[] photoByte) {
        String strFileExtendName = "jpg";
        if ((photoByte[0] == 71) && (photoByte[1] == 73) && (photoByte[2] == 70) && (photoByte[3] == 56)
                && ((photoByte[4] == 55) || (photoByte[4] == 57)) && (photoByte[5] == 97)) {
            strFileExtendName = "gif";
        } else if ((photoByte[6] == 74) && (photoByte[7] == 70) && (photoByte[8] == 73) && (photoByte[9] == 70)) {
            strFileExtendName = "jpg";
        } else if ((photoByte[0] == 66) && (photoByte[1] == 77)) {
            strFileExtendName = "bmp";
        } else if ((photoByte[1] == 80) && (photoByte[2] == 78) && (photoByte[3] == 71)) {
            strFileExtendName = "png";
        }
        return strFileExtendName;
    }

    /**
     * 获取文件名称 /profile/upload/2022/04/16/ruoyi.png -- ruoyi.png
     *
     * @param fileName 路径名称
     * @return 没有文件路径的名称
     */
    public static String getName(String fileName) {
        if (fileName == null) {
            return null;
        }
        int lastUnixPos = fileName.lastIndexOf('/');
        int lastWindowsPos = fileName.lastIndexOf('\\');
        int index = Math.max(lastUnixPos, lastWindowsPos);
        return fileName.substring(index + 1);
    }


    /**
     * 判断文件夹是否存在，不存在则创建
     *
     * @param destDirName
     */
    public static boolean CreateDir(String destDirName) {
        File dir = new File(destDirName);
        if (dir.exists()) {
            return false;
        }
        if (!destDirName.endsWith(File.separator)) {
            destDirName = destDirName + File.separator;
        }
        // 创建目录
        if (dir.mkdirs()) {
            return true;
        } else {
            return false;
        }

    }

    public static Integer getFileSize(String path) {
        File file = new File(path);

        long length = file.length();
        long filesize = length / 1024;
        BigDecimal bg = new BigDecimal(filesize);

        return bg.intValue();
    }


    //参数imgFile：图片完整路径
    public static String getImgBase64Str(String imgFile) {
        // 将图片文件转化为字节数组字符串，并对其进行Base64编码处理
        InputStream in = null;
        byte[] data = null;
        // 读取图片字节数组
        try {
            in = new FileInputStream(imgFile);
            data = new byte[in.available()];
            in.read(data);
            in.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
        return Base64.getEncoder().encodeToString(data);
    }

    /**
     * 压缩图像并保存到指定路径
     *
     * @param imgPath2    包含 Base64 编码图像数据的字符串
     * @param outputPath  指定保存压缩图像的文件路径
     * @param compression 压缩比例，范围为 0.0 到 1.0
     */

    /**
     * 视频压缩 使用 Maven仓库jave-core核心包，也是基于ffmpeg技术
     * 优点：不用安装ffmpeg，配置环境变量，pom.xml引入即可。
     *
     * @param convertFile
     * @param targetFile
     */
//    public static void videoCompression(String convertFile, String targetFile) {
//        File conFile = new File(convertFile);
//        if (conFile == null) {
//            return;
//        }
//        try {
//            File tarFile = new File(targetFile);
//            MultimediaObject multimediaObject = new MultimediaObject(conFile);
//            //根据视频大小来判断是否需要压缩
//            double maxSize = 0.5;
//            double converFileSize = Math.ceil(conFile.length() / 1024 / 1024);
//            int second = (int) multimediaObject.getInfo().getDuration();
//            BigDecimal bd = new BigDecimal(String.format("%.4f", converFileSize / second));
//            logger.info("====================开始压缩视频-->平均每秒 " + bd + "MB====================");
////            如果视频大于512k才做压缩
//            if (converFileSize > maxSize) {
//                long beginTime = System.currentTimeMillis();
//                //视频属性设置
//                int maxFrameRate = 20;
//                VideoAttributes video = new VideoAttributes();
//                video.setCodec("h264");
//                video.setFrameRate(maxFrameRate);
//                video.setBitRate(new Integer(500000));
//                //设置音频比特率,单位:b (比特率越高，清晰度/音质越好，当然文件也就越大 800000 = 800kb)
//                EncodingAttributes attr = new EncodingAttributes();
//                attr.setOutputFormat("mp4");
//                attr.setDecodingThreads(Runtime.getRuntime().availableProcessors()/2);
//                attr.setEncodingThreads(Runtime.getRuntime().availableProcessors() / 2);
//                attr.setVideoAttributes(video);
//                logger.info("===================视频压缩开始，压缩文件" + convertFile + "==================");
//                Encoder encoder = new Encoder();
//                encoder.encode(new MultimediaObject(conFile), tarFile, attr);
//                long endTime = System.currentTimeMillis();
//
//                logger.info("===================视频压缩完成，耗时：" + (endTime - beginTime) / 1000 + "S==================");
//            }
//        } catch (Exception e) {
//            e.printStackTrace();
//            logger.info("===================压缩文件出现异常：" + e.getMessage() + "===================");
//        }
//    }

    /**
     * 快速压缩视频转码函数
     * @param convertFile
     * @param targetFile
     */
    public static void videoFastCompression(String convertFile, String targetFile) {
        File conFile = new File(convertFile);
        if (conFile == null || !conFile.exists()) {
            logger.error("转码文件源不存在.");
            return;
        }
        try {
            File tarFile = new File(targetFile);
            File targetDir = tarFile.getParentFile();
            if (!targetDir.exists()) {
                if (!targetDir.mkdirs()) {
                    logger.error("转码成功，输出视频路径不存在.");
                    return;
                }
            }
            double maxSize = 0.5; // 最大文件大小阈值（单位：MB）
            double converFileSize = Math.ceil(conFile.length() / 1024.0 / 1024.0);

            // 假设你有某种方法获取视频时长（秒）
            int second = getVideoDuration(conFile); // 这个方法需要实现
            BigDecimal bd = new BigDecimal(String.format("%.4f", converFileSize / second));
            logger.info("====================开始压缩视频-->平均每秒 " + bd + "MB====================");

            if (converFileSize > maxSize) {
                long beginTime = System.currentTimeMillis();
                // FFmpeg命令
                String ffmpegCommand = String.format(
                        "ffmpeg -y -i %s -an -qscale 0 -vcodec libx264 -crf 34 -threads %d -preset ultrafast %s",
                        convertFile, Runtime.getRuntime().availableProcessors(), targetFile);

                ProcessBuilder processBuilder = new ProcessBuilder(ffmpegCommand.split(" "));
                processBuilder.redirectErrorStream(true);
                Process process = processBuilder.start();

                BufferedReader reader = new BufferedReader(new InputStreamReader(process.getInputStream()));
                String line;
                StringBuilder output = new StringBuilder();
                while ((line = reader.readLine()) != null) {
                    output.append(line).append("\n");
                    System.out.println(line);
                }

                int exitCode = process.waitFor();
                long endTime = System.currentTimeMillis();

                if (exitCode == 0) {
                    logger.info("===================视频压缩完成，耗时：" + (endTime - beginTime) / 1000 + "S==================");
                } else {
                    logger.error("FFmpeg exited with code: " + exitCode);
                    logger.error("FFmpeg output: " + output.toString());
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
            logger.error("===================压缩文件出现异常：" + e.getMessage() + "===================");
        }
    }

    private static int getVideoDuration(File file) {
        try {
            String ffprobeCommand = String.format("ffprobe -v error -show_entries format=duration -of default=noprint_wrappers=1:nokey=1 %s", file.getAbsolutePath());
            ProcessBuilder processBuilder = new ProcessBuilder(ffprobeCommand.split(" "));
            processBuilder.redirectErrorStream(true);
            Process process = processBuilder.start();

            BufferedReader reader = new BufferedReader(new InputStreamReader(process.getInputStream()));
            String line = reader.readLine();

            process.waitFor();
            return (int) Math.ceil(Double.parseDouble(line));
        } catch (Exception e) {
            e.printStackTrace();
            return 0;
        }
    }
    public static void main(String[] args) {
        // 示例用法
//        String imgPath2 = "Base64 编码的图像数据"; // 替换为实际的 Base64 编码数据
//        String outputPath = "path/to/compressed/image.jpg"; // 替换为实际的输出路径
//        double compression = 0.8; // 设置压缩比例
//
//        compressImage(imgPath2, outputPath, compression);

//        String targetPath = file.getPath().substring(0, file.getPath().lastIndexOf("\\")) + "\\Compression\\" +
//                file.getName();
//        videoCompression("E:/video/12341.mp4","E:/video/Compression/12341.mp4");
//        Integer fileSize = getFileSize("D:/video/20240409/Compression/240112630002N0025639_F1E_1.mp4");
//        System.out.println(fileSize);
        String convertFile = "D:/video/240426650002N0297182_L2_1.mp4";
        String targetFile = "D:/video/jiazaio.mp4";
//        videoCompression(convertFile, targetFile);
        videoFastCompression(convertFile, targetFile);
    }
}
