package com.jjkj.video.service.impl;

import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.jjkj.util.AjaxResult;
import com.jjkj.util.FileUtils;
import com.jjkj.video.mapper.DataResultVideoMapper;
import com.jjkj.video.pojo.DataResultVideo;
import com.jjkj.video.service.IDataResultVideoService;
import org.apache.commons.lang.StringUtils;
import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.io.File;
import java.text.ParseException;
import java.text.SimpleDateFormat;

import java.time.Duration;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.time.format.DateTimeParseException;
import java.util.*;

import static com.jjkj.hikvison.HikvisionUtil.downloadVideo;
import static com.jjkj.util.FileUtils.videoFastCompression;


/**
 * 检测视频Service业务层处理
 *
 * @author 南若飞
 * @date 2024-04-26
 */
@Service
public class DataResultVideoServiceImpl extends ServiceImpl<DataResultVideoMapper, DataResultVideo> implements IDataResultVideoService {
    @Autowired
    private DataResultVideoMapper dataResultVideoMapper;
    private Logger logger = Logger.getLogger(DataResultVideoServiceImpl.class);
    private static final int DOWNLOAD_DELAY = -5; // 5 minutes delay
    @Value("${server.ip}")
    private String serverIp;
    @Value("${server.videoPort}")
    private String port;
    @Value("${server.path}")
    private String videoPath;


    public void downloadJcsp(String params) {
        // 创建一个 DateTimeFormatter 对象，指定日期格式
        SimpleDateFormat formatter = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        String day = params.split(",")[0];
        Integer number = Integer.parseInt(params.split(",")[1]);
        List<DataResultVideo> dataVehicleVideoList = this.list(
                new QueryWrapper<DataResultVideo>()
                        .ne("download_flag", "1")
                        .or().isNull("download_flag")
                        .isNotNull("kssj")
                        .isNotNull("jssj")
                        .gt("jssj", com.jjkj.util.DateUtil.getBeforeHarfMonth(Integer.parseInt(day)))
                        .orderByDesc("jssj"));
        if (dataVehicleVideoList != null && dataVehicleVideoList.size() > 0) {
            int count = dataVehicleVideoList.size() > number ? number : dataVehicleVideoList.size();
            logger.info("下载视频条数：" + count + "条，执行时间：" + DateUtil.format(new Date(), "yyyy-MM-dd HH:mm:ss"));
//            System.out.println("下载视频条数：" + count + "条，执行时间：" + DateUtil.format(new Date(), "yyyy-MM-dd HH:mm:ss"));

            for (int i = 0; i < count; i++) {
                // 获取车辆检测结束时间
                // 将字符串解析为 LocalDate 对象
                try {
                    Date detectionEndTime = formatter.parse(dataVehicleVideoList.get(i).getJssj());
                    // 判断是否超过当前时间加上五分钟后的时间
                    if (isDetectionEndTimeBeyondFiveMinutes(detectionEndTime)) {
                        downloadJcxmVideo(dataVehicleVideoList.get(i));
                    }
                } catch (ParseException e) {
                    throw new RuntimeException(e);
                }

            }
        }


        //可以解析参数，设置下载多少天前的车，每次下载多少条记录
        // 只下载没有退办且已下线的车辆视频
//        List<DataVehicleFlow> dataVehicleFlowList = this.iDataVehicleFlowService.list(
//                new QueryWrapper<DataVehicleFlow>()
//                        .ne("flag", "1")
//                        .or().isNull("flag")
//                        .ne("lszt", "90")
//                        .isNotNull("kssj")
//                        .isNotNull("jssj")
//                        .gt("createtime", com.jjkj.util.DateUtil.getBeforeHarfMonth(Integer.parseInt(day)))
//                        .orderByDesc("createtime"));
//        if (dataVehicleFlowList != null && dataVehicleFlowList.size() > 0) {
//            int count = dataVehicleFlowList.size() > number ? number : dataVehicleFlowList.size();
//            logger.info("下载视频车辆数：" + count + "辆，执行时间：" + DateUtil.format(new Date(), "yyyy-MM-dd HH:mm:ss"));
//            for (int i = 0; i < count; i++) {
//                // 获取车辆检测结束时间
//                Date detectionEndTime = dataVehicleFlowList.get(i).getJssj();
//                // 判断是否超过当前时间加上五分钟后的时间
//                if (isDetectionEndTimeBeyondFiveMinutes(detectionEndTime)) {
//                    downloadVehicleVideo(dataVehicleFlowList.get(i));
//                }
//            }
//            for (DataVehicleFlow dataVehicleFlow : dataVehicleFlowList) {
//                // 获取车辆检测结束时间
//                Date detectionEndTime = dataVehicleFlow.getJssj();
//                // 判断是否超过当前时间加上五分钟后的时间
//                if (isDetectionEndTimeBeyondFiveMinutes(detectionEndTime)) {
//                    downloadVehicleVideo(dataVehicleFlow);
//                }
//            }
//        }
    }

    /**
     * 根据检测项目下载视频
     *
     * @param dataResultVideo
     */
    private void downloadJcxmVideo(DataResultVideo dataResultVideo) {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        boolean flag = false;
        String savePath = videoPath + "/" + DateUtil.format(new Date(), "yyyyMMdd");
        String videoName = dataResultVideo.getLsh() + "_" + dataResultVideo.getSpbh() + "_" + dataResultVideo.getJycs() + ".mp4";
        try {
            if (StringUtils.isNotEmpty(dataResultVideo.getIp()) && StringUtils.isNotEmpty(dataResultVideo.getPort())
                    && StringUtils.isNotEmpty(dataResultVideo.getName()) && StringUtils.isNotEmpty(dataResultVideo.getChannel())
                    && StringUtils.isNotEmpty(dataResultVideo.getPassword()) && StringUtils.isNotEmpty(dataResultVideo.getKssj())
                    && StringUtils.isNotEmpty(dataResultVideo.getJssj())) {
                if (JcTimeValid(dataResultVideo)) {
                    // 超出240秒，截取4分钟视频
                    dataResultVideo.setJssj(getTimeAfterFourMinutes(dataResultVideo.getKssj()));
                }
                flag = downloadVideo(dataResultVideo.getIp(), savePath, videoName, Short.parseShort(dataResultVideo.getPort()),
                        dataResultVideo.getName(), Integer.parseInt(dataResultVideo.getChannel()), dataResultVideo.getPassword(), sdf.parse(dataResultVideo.getKssj()),
                        sdf.parse(dataResultVideo.getJssj()));


            }
            if (flag) {
                String savepath = savePath + "/Compression/" + videoName;
                videoFastCompression(savePath + "/" + videoName, savepath);
                //下载成功，修改下载状态
                dataResultVideo.setAbsPath(savepath);
                dataResultVideo.setRelPath("http://" + serverIp + ":" + port + "/carcheck_videos/" + DateUtil.format(new Date(), "yyyyMMdd") + "/Compression/" + videoName);
                //计算文件大小
                dataResultVideo.setFilesize(String.valueOf(FileUtils.getFileSize(savepath)));
                //判断下载状态,排除掉小于512kb和无文件的视频
                File file = new File(savepath);
                int size = FileUtils.getFileSize(savepath);
                dataResultVideo.setFilesize(String.valueOf(size));
                if (file.exists() && size > 514) {
                    dataResultVideo.setDownloadFlag("1");
                } else {
                    dataResultVideo.setDownloadFlag("0");
                }
                int i = this.dataResultVideoMapper.updateDataResultVideo(dataResultVideo);
                logger.info("视频下载信息@流水号：" + dataResultVideo.getLsh() + "项目:" + dataResultVideo.getJyxm() + "" + "视频更新状态：" + i);
//                System.out.println("视频下载信息@流水号：" + dataResultVideo.getLsh() + "项目:" + dataResultVideo.getJyxm() + "" + "视频更新状态：" + i);
            }

        } catch (ParseException e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * 判断结束时间是否比开始时间超过240秒
     *
     * @param dataResultVideo
     * @return
     */
    private boolean JcTimeValid(DataResultVideo dataResultVideo) {
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
        try {
            LocalDateTime dateTime1 = LocalDateTime.parse(dataResultVideo.getKssj(), formatter);
            LocalDateTime dateTime2 = LocalDateTime.parse(dataResultVideo.getJssj(), formatter);
            Duration duration = Duration.between(dateTime1, dateTime2);
            long differenceInMinutes = Math.abs(duration.toMinutes());
            return differenceInMinutes > 4;

        } catch (DateTimeParseException e) {
            e.printStackTrace();
            return false;
        }

    }

    /**
     * 获取240秒后的时间
     *
     * @param dateStr
     * @return
     */
    public static String getTimeAfterFourMinutes(String dateStr) {
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
        try {
            LocalDateTime dateTime = LocalDateTime.parse(dateStr, formatter);
            LocalDateTime dateTimePlusFourMinutes = dateTime.plusMinutes(4);
            return dateTimePlusFourMinutes.format(formatter);
        } catch (DateTimeParseException e) {
            e.printStackTrace();
            return null;
        }
    }

    public static void main(String[] args) {
        String dateStr = "2024-05-20 08:45:00";
        String newDateStr = getTimeAfterFourMinutes(dateStr);
        System.out.println("原始时间: " + dateStr);
        System.out.println("四分钟后的时间: " + newDateStr);
    }

    // 判断检测结束时间是否超过当前时间减去五分钟后的时间
    private boolean isDetectionEndTimeBeyondFiveMinutes(Date detectionEndTime) {
        Date beforeTime = calculateFutureTime();
        return detectionEndTime.before(beforeTime);
    }

    // 计算当前时间减去五分钟后的时间
    private Date calculateFutureTime() {
        Calendar calendar = Calendar.getInstance(TimeZone.getTimeZone("Asia/Shanghai"));
        calendar.setTime(new Date());
        calendar.add(Calendar.MINUTE, DOWNLOAD_DELAY);
        return calendar.getTime();
    }

    /**
     * 下载车辆检测视频
     */
//    private void downloadVehicleVideo(DataVehicleFlow dataVehicleFlow) {
//        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
//        List<DataResultVideo> videoList = this.list(new QueryWrapper<DataResultVideo>().isNotNull("kssj")
//                .isNotNull("jssj").eq("lsh", dataVehicleFlow.getLsh()).ne("download_flag", "1"));
//        if (videoList != null && videoList.size() > 0) {
//            boolean flag = false;
//            for (DataResultVideo dataResultVideo : videoList) {
//                String savePath = videoPath + "/" + DateUtil.format(new Date(), "yyyyMMdd");
//                String videoName = dataResultVideo.getLsh() + "_" + dataResultVideo.getSpbh() + "_" + dataResultVideo.getJycs() + ".mp4";
//                try {
//                    if (StringUtils.isNotEmpty(dataResultVideo.getIp()) && StringUtils.isNotEmpty(dataResultVideo.getPort())
//                            && StringUtils.isNotEmpty(dataResultVideo.getName()) && StringUtils.isNotEmpty(dataResultVideo.getChannel())
//                            && StringUtils.isNotEmpty(dataResultVideo.getPassword()) && StringUtils.isNotEmpty(dataResultVideo.getKssj())
//                            && StringUtils.isNotEmpty(dataResultVideo.getJssj())) {
//                        flag = downloadVideo(dataResultVideo.getIp(), savePath, videoName, Short.parseShort(dataResultVideo.getPort()),
//                                dataResultVideo.getName(), Integer.parseInt(dataResultVideo.getChannel()), dataResultVideo.getPassword(), sdf.parse(dataResultVideo.getKssj()),
//                                sdf.parse(dataResultVideo.getJssj()));
//
//                    }
//                    if (flag) {
//                        String savepath = savePath + "/Compression/" + videoName;
//                        videoFastCompression(savePath + "/" + videoName, savepath);
//                        //下载成功，修改下载状态
//                        dataResultVideo.setAbsPath(savepath);
//                        dataResultVideo.setRelPath("http://" + serverIp + ":" + port + "/carcheck_videos/" + DateUtil.format(new Date(), "yyyyMMdd") + "/Compression/" + videoName);
//                        //计算文件大小
//                        dataResultVideo.setFilesize(String.valueOf(FileUtils.getFileSize(savepath)));
//                        //判断下载状态,排除掉小于512kb和无文件的视频
//                        File file = new File(savepath);
//                        int size = FileUtils.getFileSize(savepath);
//                        dataResultVideo.setFilesize(String.valueOf(size));
//                        if (file.exists() && size > 514) {
//                            dataResultVideo.setDownloadFlag("1");
//                        } else {
//                            dataResultVideo.setDownloadFlag("0");
//                        }
//                        int i = this.dataResultVideoMapper.updateDataResultVideo(dataResultVideo);
//                        System.out.println("视频表修改状态：" + i);
//                    }
//
//                } catch (ParseException e) {
//                    throw new RuntimeException(e);
//                }
//
//            }
//        } else {
////            未查询到视频记录的 流水表更新为已下载
//            dataVehicleFlow.setFlag("1");
//            boolean update = this.iDataVehicleFlowService.update(dataVehicleFlow, new QueryWrapper<DataVehicleFlow>().eq("lsh", dataVehicleFlow.getLsh()).eq("jycs", dataVehicleFlow.getJycs()));
//            System.out.println("流水表修改状态：" + update);
//        }
//
//
//    }
    @Override
    public AjaxResult countJcsp() {
        AjaxResult ajaxResult = new AjaxResult();
        // 获取当前时间
        Calendar calendar = Calendar.getInstance();
        // 将时、分、秒和毫秒设置为0
        calendar.set(Calendar.HOUR_OF_DAY, 0);
        calendar.set(Calendar.MINUTE, 0);
        calendar.set(Calendar.SECOND, 0);
        calendar.set(Calendar.MILLISECOND, 0);
        // 格式化日期
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        String formatToday = sdf.format(calendar.getTime());
        QueryWrapper queryWrapper = new QueryWrapper<DataResultVideo>();
        queryWrapper.eq("download_flag", "1");
        queryWrapper.isNotNull("kssj");
        queryWrapper.isNotNull("jssj");
        List<DataResultVideo> videoList = this.list(queryWrapper);
        if (videoList != null) {
            // 下载总数
            ajaxResult.put("sumNum", videoList.size());
        }
        queryWrapper.gt("jssj", formatToday);
        List<DataResultVideo> todayVideoList = this.list(queryWrapper);
        if (todayVideoList != null) {
            // 下载总数
            ajaxResult.put("todayNum", todayVideoList.size());
        }
        return ajaxResult;
    }

}
